#ifndef _IPL98_CAMERAFIREWIREUMC_H
#define _IPL98_CAMERAFIREWIREUMC_H

#include "camera_ipl98_setup.h" /* always include the camera add-on library setup file */

#include <string>
#include <vector>
#include <strstream>
#include <istream>
#include <stdio.h>
#include <windows.h>
#include "1394Camera.h"
#include <deque>



//using ipl::UINT16;
//using ipl::CStdImage;
//using ipl::CImage;
//using ipl::CByteImage;
using std::string;
using std::vector;
using std::strstream;
using std::istream;
using std::pair;
using std::deque;
using std::ifstream;
using std::ofstream;

/** Based on the <a href="http://www.cs.cmu.edu/~iwan/1394">"CMU IEEE-1394 Digital Camera Windows Driver"</a>. 
	In order to use the class, you must link with the supplied library-file (1394camera.lib).

	Some examples on how to use this class:
	\verbatim

	bool selected=false, initialized=false;
	ipl::CCameraFirewireCMU theCamera;
	theCamera.QueryCameras();
	
	selected = theCamera.SelectCamera(0);
	if(selected){
		initialized = theCamera.Initialize();
		if(initialized)
		{		
			cout<< theCamera.GetCameraVendor() << " " << theCamera.GetCameraName() << endl; 

			if (!theCamera.AutoExposureAvailable())
				cout << "AutoExposure not available"<< endl;
			else{
				cout << "AutoExposure available"<< endl;
				cout << "GetAutoExposure(): " << theCamera.GetAutoExposure()<< endl;
				cout << "GetAutoExposureMin(): " <<  theCamera.GetAutoExposureMin()<< endl;
				cout << "GetAutoExposureMax(): " <<  theCamera.GetAutoExposureMax()<< endl;
				cout << "SetAutoExposure(117) " << theCamera.SetAutoExposure(117)<< endl;
				cout << "GetAutoExposure(): " << theCamera.GetAutoExposure()<< endl;
			}

			cout << "GetAllModes:" << endl;
			ostringstream os;
			theCamera.GetAllFormatsAndModes(os);
			cout << os.str() << endl;

			vector<string> CamInfo;
			cout << "GetAllCameraInfo(CamInfo);" << endl;
			theCamera.GetAllCameraInfo(CamInfo);
			for(int i=0; i<theCamera.GetTotalCameras(); i++)
			{
				cout << CamInfo[i] << endl;
			}
		}
	}
	cout << "Finished..." << endl;
	\endverbatim
	Here is how to actually grab an image, ensure that you set the format, mode and framerate
	to values supported by the camera:
	\verbatim

	bool selected=false, initialized=false;
	ipl::CCameraFirewireCMU theCamera;
	theCamera.QueryCameras();
	
	selected = theCamera.SelectCamera(0);
	if(selected){
		initialized = theCamera.Initialize();
		if(initialized)
		{
			theCamera.SetFormatModeFrameRate(0,5,15);

			CImage Img;
			theCamera.Acquire(Img);
			Img.Save("d:/temp/test.bmp");
		}
	}
	\endverbatim
	This class also works with several instances attached to different cameras:
	\verbatim
	CCameraFirewireCMU Cam1,Cam2;
	unsigned int NrOfCams=Cam1.QueryCameras();
	cout << "Total cameras found: " << NrOfCams << endl;
	if (NrOfCams>=2)
	{
		vector<string> CamInfo;
		Cam1.GetAllCameraInfo(CamInfo);
		for(int i=0;i<CamInfo.size();i++)
		{
			cout << CamInfo[i] << endl;
		}
		// these calls to Load assumes a camera setup file is available
		// from a previous successful call to CCameraFirewireCMU::Save(...)
		// and it assumes that the cameras defined in the setup files are
		// present on the firewire bus
		if ((Cam1.Load("d:/temp/cam_settings.cfg",true)==true) &&
			(Cam2.Load("d:/temp/cam_settings2.cfg",true)==true))
		{
			CImage Img;
			Cam1.Acquire(Img);
			Img.Save("d:/temp/test1.bmp");
			Cam2.Acquire(Img);
			Img.Save("d:/temp/test2.bmp");
		}
	}
	\endverbatim
	Using the camera in extended mode is the only way to reach maximum framerates. The drawback
	is that the next image in the buffer may be old. Here
	is an example on grabbing images in extended mode:
	\verbatim
	CCameraFirewireCMU Cam;
	Cam.QueryCameras();
	if (Cam.SelectCamera(0)==true)
	{
		Cam.Initialize();
		// set a format, mode and framerate supported by current camera
		Cam.SetFormatModeFrameRate(0,1,30);
		Cam.StartImageAcquisitionExtended();
		unsigned int TotalImages=100;
		CImage Img;
		CTimeDate Timer;
		Timer.StartTimer();
		for(int i=0;i<TotalImages;i++)
		{
			Cam.AcquireExtended(Img);
			// do something with Img
			// ...
		}
		double Time=Timer.StopTimer();
		Cam.StopImageAcquisitionExtended();
		cout << "Framerate = " << TotalImages/Time << endl;
	}
	\endverbatim

	Note: With the current implementation of the CMU IEEE-1394 Digital Camera Windows Driver, we
	are not able to retrieve the current format, mode and framerate settings in the camera when
	connecting to it. So if you need to reuse previous format/mode/framerate settings, you need 
	to save a setup file and always make sure to read that file next time the camera is used.

	To-do list:
	\verbatim
	* The way QueryCameras() works is not optimal. It allocates memory on the 
	  heap for the static member m_ConnectedCams which leads to memory leaks, 
	  not a serious one though. The behaviour of the method is also the reason
	  for the introduction of m_CamObjects, hopefully we can get rid of this
	  in a future version.
	* We have some problems with Format 7 and the Basler cameras. As long as we use the maximum
	  resolution in Format 7 with those cameras, everything works well, but if we select a
	  smaller area, we get incorrect images or the acquisition fails.
	\endverbatim
	This class is part of the <a href="http://www.mip.sdu.dk/ipl98">IPL98 Camera add-on library </a>.
	\class CCameraFirewireCMU camera/camera_firewire/camera_firewire_cmu.h
	@version 0.50
	@author Jon Grove Hermansen (grove@mip.sdu.dk) & Bo Kehlet Nielsen (kehlet@mip.sdu.dk)
*/
class CCameraFirewireCMU
{
protected: // structs/classes
	class CAMERA_STRUCT
	{
	public:
		/// The handle to the camera
		C1394Camera* m_phCamera;
		/// The camera vendor description
		string m_VendorName;
		/// The camera model description
		string m_ModelName;
		/// The camera serial number
		unsigned long int m_SerialNumber;
	};
	
	
	bool m_Initialized;
	/// Gain factor
	double m_GainValue;
	/// Gain min and max values
	double m_GainMin, m_GainMax;
	/// Shutter value in micro-seconds
	double m_ShutterValue;
	/// Shutter min and max values in micro seconds
	double m_ShutterMin, m_ShutterMax;
	/// Image width and height
	unsigned int m_Width, m_Height;
	/** True when Initialize() or another derived class implementation of  a Initialize(..)
	method succeeds. IsInitialized() can be used to check this value. */

	
	
protected: // attributes
	/** Current version of this class, defined in camera_firewire_cmu.cpp. */
	static const double m_Version;
	/** A delay where the process sleeps is sometimes necessary, some cameras need this
		delay in some cases, otherwise they will respond with weird error messages! */
	static const unsigned int CAM_DELAY_SLEEP;
	/// Set to true by Load() or Initialize(). Initialized to false in constructor
	bool m_SettingsInitialized;
	/// Current AutoExposure values
	bool m_AutoExposureOn;
	bool m_AutoExposureAuto;
	bool m_AutoExposurePresent;
	double m_AutoExposureValue;
	double m_AutoExposureMax;
	double m_AutoExposureMin;
	
	/// Current Brightness values
	bool m_BrightnessOn;
	bool m_BrightnessAuto;
	bool m_BrightnessPresent;
	double m_BrightnessValue;
	double m_BrightnessMin;
	double m_BrightnessMax;
	
	/// Current Gamme values
	bool m_GammaOn;
	double m_GammaValue;
	double m_GammaAuto;
	bool m_GammaPresent;
	double m_GammaMax;
	double m_GammaMin;
	
	/// Current Saturation values
	bool m_SaturationOn;
	bool m_SaturationAuto;
	bool m_SaturationPresent;
	double m_SaturationValue;
	double m_SaturationMin;
	double m_SaturationMax;
	
	/// Current Sharpness values
	bool m_SharpnessOn;
	bool m_SharpnessAuto;
	bool m_SharpnessPresent;
	double m_SharpnessValue;
	double m_SharpnessMin;
	double m_SharpnessMax;
	
	/// Current WhiteBalance values
	bool m_WhiteBalanceOn;
	bool m_WhiteBalanceAuto;
	bool m_WhiteBalancePresent;
	double m_WhiteBalanceUB;
	double m_WhiteBalanceVR;
	double m_WhiteBalanceMin;
	double m_WhiteBalanceMax;
	
	/// Current Hue	values
	bool m_HueOn;
	double m_HueValue;
	bool m_HueAuto;
	bool m_HuePresent;
	double m_HueMin;
	double m_HueMax;
	
	/// Current Iris values
	bool m_IrisOn;
	bool m_IrisAuto;
	bool m_IrisPresent;
	double m_IrisValue;
	double m_IrisMin;
	double m_IrisMax;
	
	/// Current Focus values
	bool m_FocusOn;
	bool m_FocusPresent;
	bool m_FocusAuto;
	double m_FocusValue;
	double m_FocusMin;
	double m_FocusMax;
	
	/// Current Zoom values
	bool m_ZoomOn;
	bool m_ZoomPresent;
	bool m_ZoomAuto;
	double m_ZoomValue;
	double m_ZoomMin;
	double m_ZoomMax;
	
	bool m_GainOn;
	bool m_GainPresent;
	bool m_GainAuto; 
	bool m_ShutterOn;
	bool m_ShutterPresent;
	bool m_ShutterAuto;
   
    /// Set to true, when InitializeAllCameras() has been successfully called. */
    static bool m_AllCamerasInitialized;
    /// Total number of cameras, set by InitializeAllCameras().
    static unsigned int m_TotalCams;
    /** Set to true, when InitializeAllCameras() has been successfully called. */
    static bool m_TotalCamsAvailable;
    /// Set when calling InitializeAllCameras, containes a CAMERA_STRUCT for each connected camera
    static vector<CAMERA_STRUCT*> m_ConnectedCams;
	
	/** A list of all objects allocated. The constructor will add a pointer to itself
		when a new object is constructed, and the destructor will remove that object.
		This is needed when calling QueryCameras() more than one time, since we need
		to stop all cameras first. */
	static vector<CCameraFirewireCMU*> m_CamObjects;

	/// Number of Bits pr. pixel
    unsigned int m_BitsPerPixel;
	// The current Format
	unsigned int m_Format;
	/// The current Mode
	unsigned int m_Mode;
	/// Used if format is 7
	
	
	// for partial scan?
	//CPoint2D<UINT16> m_UL, m_LR;

	/// The current Framerate index
	unsigned int m_FrameRate;
	/// Set to true when camera has partial scan feature
	bool m_PartialScanAvailable;	
	/// The Partial scan modes that are available
	bool m_PartialScanModes[8];
	/// The Formats that are available
	bool m_FormatsAvailable[3];
	/// The modes that are available
	bool m_ModesAvailable[3][8];
	/// String representation of color formats for each format and mode
	static const string sColorFormat[3][8];
	/// Color codings for format 7
	static const unsigned int Format7ColorCoding[7];
	/// String representation of color codings for format 7
	static const string sFormat7ColorCoding[7];
	/// String representations of image sizes for each format and mode
	static const string sImageSize[3][8];
	/// Image sizes for each format and mode
	static const unsigned int ImageSize[3][8][2];
	/// Bits per pixel for each format [0..2] and mode
	static const unsigned int BitsPerPixelTable[3][8];
	/// String representations of frame rates 
	static const string sFrameRates[6];
	/// Frame rates
	static const double FrameRates[6];
	/// If set to true, image acquisition has been started (internal use)
	bool m_AcquireStarted;
	/// If set to true, extended capture has begun
	bool m_ExtendedAcquireStarted;
	/// If set to true, a camera has been selected in middleware
	bool m_Selected;
	/// if set to true, link has been checked, and each camera has been queried for name information
	static bool m_QueryCameras;
	/// Only used temporarely when streaming in from a file, by edr@mip.sdu.dk
	string m_VendorName;
	/// Only used temporarely when streaming in from a file, by edr@mip.sdu.dk
	string m_ModelName;
	/// Only used temporarely when streaming in from a file, by edr@mip.sdu.dk
	unsigned long int m_SerialNumber;

public: // methods
	/** Closes all internal acquisition started by a previous call to Initialize().
		IsInitialized() will hereafter return false. Owerwrite base class' definition.
		@return True if successfull, false otherwise */
	bool Close();	
  
    /// default constructor
    CCameraFirewireCMU();
    /// copy constructor
    CCameraFirewireCMU(const CCameraFirewireCMU& Source);
    /// default destructor
    ~CCameraFirewireCMU();

	/** Returns the version of this class. */
	static double GetVersion();

	/** The current camera - used for holding all setting for the actual camera. */
    C1394Camera* m_phCamera;
		
	/** @name Camera detection.
		The following methods are used for camera detection and selection. With these
		methods you can retrieve a camera handle for each camera, a handle is used
		to initialize this object. */
	//@{ 
	/** Detects all cameras on the firewire bus, handles and other information
		for each camera are stored in the static member m_ConnectedCams.
		Important: All camera object will be closed by a call to Close(). The
		user needs to manually initialize them again.
		@return Total number of found cameras. */
	static unsigned int QueryCameras();
	/** Used for initializing the camera before using it.
		@return True if successfull, false otherwise. */
	bool Initialize();
	/** Used for selecting a camera using the node number before using it. 
		@return True if successfull, false otherwise. */
	bool SelectCamera(int node);
	/** Used for selecting a camera using the serialnumber.
		@return True if successfull, false otherwise. */
	bool SelectCameraSerial(unsigned long int serial);
	/** Used for selecting a camera using the cameras name
		@return True if successfull, false otherwise. */
	bool SelectCameraName(string name);
	/** Used for checking how many cameras there are connected. Returns the number 
		of cemeras.
		@return The number of cameras connected, -1 if an error occur. */
	unsigned int GetTotalCameras();
	/** Used for getting info about all connected cameras. Returns the number 
		of cameras. The returned format is:
		\verbatim

		Vendor-name : Model-name : Serial-number
		\endverbatim
		@param CamInfo A vector with string - the info is placed here!
		@return True if successfull, false otherwise.
		@see SplitCameraInfo */
	bool GetAllCameraInfo(vector<string>& CamInfo);
	/** Splits a string retrieved from GetAllCameraInfo() into supplied strings Vendor, Model,
		and integer SerialNr. The resulting values can be used to select a specific camera
		by calling SelectCameraSerial() or SelectCameraName().
		@return False, if CamInfo has wrong format.
		@see GetAllCameraInfo */
	static bool SplitCameraInfo(const char* CamInfo, string& Vendor, string& Model, 
								unsigned long int& SerialNr);
	/** Used for getting the cameras name. */
	string GetCameraName() const;
	/** Used for getting the cameras vendor. */
	string GetCameraVendor() const;				
	/** Used for getting the cameras serialnumber (low part). */
	unsigned long int GetSerialNumber() const;
	//@}
	/** Used for saving the selected cameras settings.
		@param pPathAndFileName The path and file name of the output file
		@return True if successfull, false otherwise. */
	bool Save(const char* pPathAndFileName) const;
	/** Writes all data to stream. Format is:
		\verbatim		
		# Firewire camera setup written by CCameraFirewireCMU class
		#
		# CCameraFirewireCMU class version
		CamFirewireVersion

		# camera identification
		VendorName
		ModelName
		SerialNumber

		# camera pixel setup, resolution, and framerate
		Format
		Mode
		FrameRate

		# Firewire camera settings
		AutoExposureValue
		AutoExposureAuto
		BrightnessValue
		BrightnessAuto
		GainValue
		GainAuto
		GammaValue
		IrisValue
		IrisAuto
		PartialScanValue
		PartialScanAuto
		SaturationValue
		SharpnessValue
		SharpnessAuto
		ShutterValue
		ShutterAuto
		WhiteBalanceUB
		WhiteBalanceVR
		WhiteBalanceAuto
		ZoomValue
		\endverbatim
		For a more compact output of camera settings, only for visual inspection, see
		GetCurrentSettings(). */
	friend strstream& operator<<(strstream& os, const CCameraFirewireCMU& Cam);
	/** Loads data from given path and filename into this class by calling the friend operator>>.
		The class will initialize the camera and set parameters from file for camera.
		The values updated in this class are:
		\verbatim
			VendorName
			ModelName
			SerialNumber
			Format
			Mode
			FrameRate
			AutoExposureValue
			AutoExposureAuto
			BrightnessValue
			BrightnessAuto
			GainValue
			GainAuto
			GammaValue
			IrisValue
			IrisAuto
			PartialScanValue
			PartialScanAuto
			SaturationValue
			SharpnessValue
			SharpnessAuto
			ShutterValue
			ShutterAuto
			WhiteBalanceValue1
			WhiteBalanceValue2
			WhiteBalanceAuto
			ZoomValue
		\endverbatim
		@param pPathAndFileName Name of file including extension and relative
			or absolute path.
		@param Start If true, the camera defined in the setup file will be selected
			and initialized with settings from the setup file. The camera will be
			started as normal acquisition, not extended
			(see StartImageAcquisitionExtended() ). If this value is false, the camera
			can be activated later by calling SetWithCurrentValues().
		@return False, if file could not be opened or if the format is not correct.
		@see GetCameraVendorNameSerial
		@see SetWithCurrentValues */
	bool Load(const char* pPathAndFileName, bool Start);
	/** Reads the version, vendor, camera name, and serial number from the given setup 
		file. This can be used to check if this camera exists before actually loading
		the camera setup file.
		@param pPathAndFileName Name of file including extension and relative
			or absolute path.
		@param Version Version number for the class that saved this setup file
			returned in this parameter.
		@param Vendor Name of vendor returned in this parameter. 
		@param ModelName Camera name returned in this parameter.
		@param Serial Serial number returned in this parameter.
		@return False, if the format of the setup is incorrect. */
	bool GetCameraVendorNameSerial(const char* pPathAndFileName, double& Version,
							string& Vendor, string& ModelName, unsigned long int& Serial) const;
	/** Sets camera format, mode, framerate, shutter, gain etc. with current 
		values in this class. This method is most often used in conjunction 
		with Load(). If the user sends false as the second parameter to Load(), 
		the values read can be checked before trying to start the camera. After 
		validation and/or changing some parameters, user can call this method 
		to start camera with those values. The camera will be started as normal
		acquisition, not extended (see StartImageAcquisitionExtended() ).
		@return False, if at least one of the current values are not valid for 
			the camera.
		@version 0.5 */
	bool SetWithCurrentValues() throw(string);
	/** Returns true, if Load() has been successfully called (or the corresponding
		streaming operator. Also returns true if camera has been manually initialized
		by calling Initialize(). */
	bool SettingsInitialized()const{return m_SettingsInitialized;}
	/** Reads input from given stream into this class. Stream must be in correct format
		according to how the << strstream operator formats output.
		For more control about loading from files, see the method CCameraFirewireCMU::Save(). */
	friend istream& operator>>(istream& is, CCameraFirewireCMU& Cam);
	/** Returns current settings for this camera in given stream. If this class not
		initialized, the value "n/a" will be streamed. This method returns a more compact
		output of camera settings than the streaming operator <<.
		@return False, if camera not initialized. */
	bool GetCurrentSettings(strstream& os) const;
	/** @name Format, Modes and FrameRates.
		The following methods are used for getting and setting format, mode and framerate
		on the selected camera. */
	//@{ 
	/** Used for getting all the modes the ca era support */
		bool GetAllFormatsAndModes(strstream& os) const;
	/** Used for getting the current video mode */
    int GetMode() const;
	/** Used for getting the current video format */
    unsigned int GetFormat() const;
	/** Used for getting the current video framerate */
    double GetFrameRate() const;
	/** Used for setting the current video format and mode.
		@return True if all is successfull otherwise false! */
	bool SetFormatMode(unsigned int Format, unsigned int Mode);
	/** Used for setting the current video format, mode and framerate.
		If StartImageAcquisitionExtended() has been called before, this method will
		automatically call StopImageAcquisitionExtended().
		@return True if all is successfull otherwise false! */
	bool SetFormatModeFrameRate(unsigned int Format, unsigned int Mode, double FrameRate);
	/** Used for getting bits per pixel in specific format and mode.
		@return the number of bits per pixel. */
	unsigned int GetBitsPerPixel(unsigned int Format, unsigned int Mode) const;
	/** Used to check if a specific format and mode combination is possible on
		camera. */
	bool FormatAndModeAvailable(unsigned int Format, unsigned int Mode) const;
	/** Used to get a string representation of the given format and mode.
		@return true if combination is valid and string representation is put
		in Str. */
	bool GetFormatModeName(string& Str,unsigned int Format, unsigned int Mode) const;
	/** Used to check if a given format is available on camera. */
	bool FormatAvailable(unsigned int Format) const;
	/** Used to check if a given frame rate is available on camera for current
		format and mode. */
	bool FrameRateAvailable(double Rate) const;
	/** Used to get a string representation of the current format. */
	string GetFormatName() const;
	/** Used to get a string representation of format given by Format 
		parameter. */
	string GetFormatName(unsigned int Format) const;
	/** Used to get information of available frame rates for at given mode and 
		format.
		@return true if Format and Mode combination is valid. Vector FrameRates 
		holds all possible frame rates. */
	bool GetFrameRatesAvailable(unsigned int Format, unsigned int Mode, vector<double>& FrameRates) const;
	/** Used to get a string representation of available frame rates for the 
		currently selected mode and format. */
	bool GetFrameRatesAvailable(strstream& os) const;
	/** Used to get string representation of current mode. */
	string GetModeName() const;
	/** Returns true if current mode is a color mode. */
	inline bool IsCurrentModeColor() const;
	/** Used to get string representation of mode given by Mode parameter. */
	string GetModeName(unsigned int Mode) const;
	/** Used to get information if a mode given by Mode is valid under the 
		currently selected format. */
	bool ModeAvailable(unsigned int Mode) const;
	/** Used to set frame rate for currently selected mode and format. Argument 
		is a double value specifying the frame rate in frames per second.
		@return true if frame rate is valid, false otherwise. */
	bool SetFrameRate(double Rate);
	//@}

	/** Acquire image from camera.
		@return False if grabbing failed  */
	//bool Acquire(CStdImage& Image);
	bool CCameraFirewireCMU::Acquire(unsigned char * data);
	
	/** Used for getting the current Height of the cameras resolution
		@return The height, or -1 if unsuccessful. */
	int GetHeight();
	/** Used for getting the current Width of the cameras resolution
		@return The width, or -1 if unsuccessful. */
	int GetWidth();
	
	/** Used for getting the currently used bits per pixel for the given
		format and mode.
		@return 8 or 24. Note: Every mode with 16 bpp is converted to 24 bpp */
	inline UINT16 BitsPerPixel() const;

	/** Used for starting the extended image acquisition. Extended image acquisition
		uses a loose frame strategy. Set the number of buffers with Buffers argument.
		@return True if successful. */
	bool StartImageAcquisitionExtended();
	/** Used for stopping the extended image acquisition. 
		@return True if successful. */
	bool StopImageAcquisitionExtended();
	/// Returns true when StartImageAcquisitionExtended() has been successfully called. */
	inline bool ImageAcquisitionExtendedStarted() const{return m_ExtendedAcquireStarted;};
	/**	Used for acquiring image during extended image acquisition. 
		@return False, if image acquisition fails. */
	//bool AcquireExtended(CStdImage& Image);
	bool AcquireExtended(unsigned char * data);		

	/** @name Camera settings.
		The following methods are used for reading and changing the various 
		settings the camera support. */
	//@{ 

	// ---------------------------- AUTOEXPOSURE -------------------------
	
	/** Returns true if the camera support AutoExposure. */
	bool AutoExposureAvailable() const;
	/** Returns true if autoexposure on the camera is turned on. False if off. */
	bool GetAutoExposureOn() const;
	/** Set actual auto exposure on value. Call with true if the camera 
		should use auto exposure, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetAutoExposureOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetAutoExposureAuto() const;
	/** Set actual auto exposure auto value. Call with true if the camera 
		should adjust auto exposure, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetAutoExposureAuto(bool automan);
	/** Get actual auto exposure value. Only valid if Initialize() has been
		successfully called	and AutoExposureAvailable() is true.
		If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return AutoExposure value (no unit)  */
	double GetAutoExposure() const;
	/** Get min. auto exposure value. Only valid if Initialize() has been
		successfully called and AutoExposureAvailable() is true.
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Min. AutoExposure value. */
	double GetAutoExposureMin() const;
	/** Get max. auto exposure value. Only valid if Initialize() has been
		successfully called and AutoExposureAvailable() is true.
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. AutoExposure value. */
	double GetAutoExposureMax() const;
	/** Set auto exposure value. If the given value is not possible the 
		nearest value are choosen.
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New auto exposure value.
		@return New nearest AutoExposure value. */
	double SetAutoExposure(double Value);
	/** Increment auto exposure value to next possible value.
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New AutoExposure value, same as previous if max. value reached  */
	double IncAutoExposure();
	/** Decrement auto exposure value to next possible value.
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New AutoExposure value, same as previous if min. value reached  */
	double DecAutoExposure();
	/** Set the auto exposure value to the previous value divided by two
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New AutoExposure value, same as previous if min. value reached  */
	double AutoExposureHalf();
	/** Set the auto exposure value to the previous value times two
		Note: If auto exposure is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New AutoExposure value, same as previous if max. value reached */
	double AutoExposureDouble();

	// ---------------------------- BRIGHTNESS ---------------------------

	/** Returns true if the camera support Brightness. */
	bool BrightnessAvailable() const;
	/** Returns true if Brightness on the camera is turned on. False if off. */
	bool GetBrightnessOn() const;
	/** Set actual Brightness on value. Call with true if the camera 
		should use Brightness, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetBrightnessOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetBrightnessAuto() const;
	/** Set actual brightness auto value. Call with true if the camera 
		should adjust brightness, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetBrightnessAuto(bool automan);
	/** Get actual brightness value. Only valid if Initialize() has been
		successfully called	and BrightnessAvailable() is true.
		If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Brightness value (no unit) */
	double GetBrightness() const;
	/** Get max. brightness value. Only valid if Initialize() has been
		successfully called and BrightnessAvailable() is true.
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Brightness value. */
	double GetBrightnessMin() const;
	/** Get max. brightness value. Only valid if Initialize() has been
		successfully called and BrightnessAvailable() is true.
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Brightness value. */
	double GetBrightnessMax() const;
	/** Set brightness value. If the given value is not possible the 
		nearest value are choosen.
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New brightness value.
		@return New nearest Brightness value. */
	double SetBrightness(double Value);
	/** Increment brightness value to next possible value.
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Brightness value, same as previous if max. value reached */
	double IncBrightness();
	/** Decrement brightness value to next possible value.
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Brightness value, same as previous if min. value reached */
	double DecBrightness();		
	/** Set the brightness value to the previous value divided by two
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Brightness value, same as previous if min. value reached */
	double BrightnessHalf();
	/** Set the brightness value to the previous value times two
		Note: If brightness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Brightness value, same as previous if max. value reached */
	double BrightnessDouble();
	
	// ---------------------------- GAMMA --------------------------------

	/** Returns true if the camera support Gamma. */
	bool GammaAvailable() const;
	/** Returns true if Gamma on the camera is turned on. False if off. */
	bool GetGammaOn() const;
	/** Set actual Gamma on value. Call with true if the camera 
		should use Gamma, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetGammaOn(bool onoff);
	/** Get actual gamma value. Only valid if Initialize() has been
		successfully called	and GammaAvailable() is true.
		If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Gamma value (no unit) */
	double GetGamma() const;
	/** Get max. gamma value. Only valid if Initialize() has been
		successfully called and GammaAvailable() is true.
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Gamma value. */
	double GetGammaMin() const;
	/** Get max. gamma value. Only valid if Initialize() has been
		successfully called and GammaAvailable() is true.
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Gamma value. */
	double GetGammaMax() const;
	/** Set gamma value. If the given value is not possible the 
		nearest value are choosen.
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New gamma value.
		@return New nearest Gamma value. */
	double SetGamma(double Value);
			/** Increment gamma value to next possible value.
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gamma value, same as previous if max. value reached */
	double IncGamma();
	/** Decrement gamma value to next possible value.
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gamma value, same as previous if min. value reached */
	double DecGamma();
	/** Set the gamma value to the previous value divided by two
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gamma value, same as previous if min. value reached */
	double GammaHalf();
	/** Set the gamma value to the previous value times two
		Note: If gamma is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gamma value, same as previous if max. value reached */
	double GammaDouble();

	// ---------------------------- SATURATION ---------------------------

	/** Returns true if the camera support Sharpness. */
	bool SaturationAvailable() const;
	/** Returns true if Saturation on the camera is turned on. False if off. */
	bool GetSaturationOn() const;
	/** Set actual Saturation on value. Call with true if the camera 
		should use Saturation, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetSaturationOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetSaturationAuto() const;
	/** Set actual sharpness auto value. Call with true if the camera 
		should adjust sharpness, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetSaturationAuto(bool automan);
	/** Get actual sharpness value. Only valid if Initialize() has been
		successfully called	and SaturationAvailable() is true.
		If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Sharpness value (no unit) */
	double GetSaturation() const;
	/** Get max. sharpness value. Only valid if Initialize() has been
		successfully called and SaturationAvailable() is true.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Sharpness value. */
	double GetSaturationMin() const;
	/** Get max. sharpness value. Only valid if Initialize() has been
		successfully called and SaturationAvailable() is true.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Sharpness value. */
	double GetSaturationMax() const;
	/** Set sharpness value. If the given value is not possible the 
		nearest value are choosen.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New sharpness value.
		@return New nearest Sharpness value. */
	double SetSaturation(double Value);
	/** Increment sharpness value to next possible value.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if max. value reached */
	double IncSaturation();
	/** Decrement sharpness value to next possible value.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if min. value reached  */
	double DecSaturation();
	/** Set the sharpness value to the previous value divided by two
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if min. value reached */
	double SaturationHalf();
	/** Set the sharpness value to the previous value times two
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if max. value reached */
	double SaturationDouble();

	// ---------------------------- SHARPNESS ----------------------------

	/** Returns true if the camera support Sharpness. */
	bool SharpnessAvailable() const;
	/** Returns true if Sharpness on the camera is turned on. False if off. */
	bool GetSharpnessOn() const;
	/** Set actual Sharpness on value. Call with true if the camera 
		should use Sharpness, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetSharpnessOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetSharpnessAuto() const;
	/** Set actual sharpness auto value. Call with true if the camera 
		should adjust sharpness, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetSharpnessAuto(bool automan);
	/** Get actual sharpness value. Only valid if Initialize() has been
		successfully called	and SharpnessAvailable() is true.
		If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Sharpness value (no unit) */
	double GetSharpness() const;
	/** Get max. sharpness value. Only valid if Initialize() has been
		successfully called and SharpnessAvailable() is true.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Sharpness value. */
	double GetSharpnessMin() const;
	/** Get max. sharpness value. Only valid if Initialize() has been
		successfully called and SharpnessAvailable() is true.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Sharpness value. */
	double GetSharpnessMax() const;
	/** Set sharpness value. If the given value is not possible the 
		nearest value are choosen.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New sharpness value.
		@return New nearest Sharpness value. */
	double SetSharpness(double Value);
	/** Increment sharpness value to next possible value.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if max. value reached */
	double IncSharpness();
	/** Decrement sharpness value to next possible value.
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if min. value reached */
	double DecSharpness();
	/** Set the sharpness value to the previous value divided by two
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if min. value reached */
	double SharpnessHalf();
	/** Set the sharpness value to the previous value times two
		Note: If sharpness is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Sharpness value, same as previous if max. value reached */
	double SharpnessDouble();

	// ---------------------------- WHITEBALANCE -------------------------

	/** Returns true if the camera support WhiteBalance.*/
	bool WhiteBalanceAvailable() const;
	/** Returns true if WhiteBalance on the camera is turned on. False if off. */
	bool GetWhiteBalanceOn() const;
	/** Set actual WhiteBalance on value. Call with true if the camera 
		should use WhiteBalance, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetWhiteBalanceOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetWhiteBalanceAuto() const;
	/** Set actual whitebalance auto value. Call with true if the camera 
		should adjust whitebalance, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetWhiteBalanceAuto(bool automan);
	/** Get actual whitebalance value. Only valid if Initialize() has been
		successfully called	and WhiteBalanceAvailable() is true.
		If whitebalance is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return A pair with the WhiteBalance values (UB, VR) */
	pair<double, double> GetWhiteBalance() const;
	/** Get max. whitebalance value. Only valid if Initialize() has been
		successfully called and WhiteBalanceAvailable() is true.
		Note: If whitebalance is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. WhiteBalance value. */
	double GetWhiteBalanceMin() const;
	/** Get max. whitebalance value. Only valid if Initialize() has been
		successfully called and WhiteBalanceAvailable() is true.
		Note: If whitebalance is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. WhiteBalance value. */
	double GetWhiteBalanceMax() const;
	/** Set whitebalance value. If the given value is not possible the 
		nearest value are choosen.
		Note: If whitebalance is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param UB the new UB value
		@param VR the new VR value
		@return A pair with the WhiteBalance values (UB, VR) */
	pair<double, double> SetWhiteBalance(double UB, double VR);
	/** Increment whitebalance value to next possible value.
		Note: If whitebalance is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New WhiteBalance value, same as previous if max. value reached  */

	// ---------------------------- HUE ----------------------------------

	/** Returns true if the camera support Hue.*/
	bool HueAvailable() const;
	/** Returns true if hue on the camera is turned on. False if off. */
	bool GetHueOn() const;
	/** Set actual hue on value. Call with true if the camera 
		should use hue, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetHueOn(bool onoff);
	/** Get actual hue value. Only valid if Initialize() has been
		successfully called	and HueAvailable() is true.
		If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Hue value (no unit) */
	double GetHue() const;
	/** Get max. hue value. Only valid if Initialize() has been
		successfully called and HueAvailable() is true.
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Hue value. */
	double GetHueMin() const;
	/** Get max. hue value. Only valid if Initialize() has been
		successfully called and HueAvailable() is true.
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Hue value. */
	double GetHueMax() const;
	/** Set hue value. If the given value is not possible the 
		nearest value are choosen.
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New hue value.
		@return New nearest Hue value. */
	double SetHue(double Value);
	/** Increment hue value to next possible value.
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Hue value, same as previous if max. value reached */
	double IncHue();
	/** Decrement hue value to next possible value.
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Hue value, same as previous if min. value reached */
	double DecHue();
	/** Set the hue value to the previous value divided by two
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Hue value, same as previous if min. value reached */
	double HueHalf();
	/** Set the hue value to the previous value times two
		Note: If hue is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Hue value, same as previous if max. value reached */
	double HueDouble();

	// ---------------------------- IRIS ---------------------------------

	/** Returns true if the camera support Iris. */
	bool IrisAvailable() const;
	/** Returns true if iris on the camera is turned on. False if off. */
	bool GetIrisOn() const;
	/** Set actual iris on value. Call with true if the camera 
		should use iris, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetIrisOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetIrisAuto() const;
	/** Set actual iris auto value. Call with true if the camera 
		should adjust iris, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetIrisAuto(bool automan);
	/** Get actual iris value. Only valid if Initialize() has been
		successfully called	and IrisAvailable() is true.
		If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Iris value (no unit) */
	double GetIris() const;
	/** Get max. iris value. Only valid if Initialize() has been
		successfully called and IrisAvailable() is true.
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Iris value. */
	double GetIrisMin() const;
	/** Get max. iris value. Only valid if Initialize() has been
		successfully called and IrisAvailable() is true.
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Iris value. */
	double GetIrisMax() const;
	/** Set iris value. If the given value is not possible the 
		nearest value are choosen.
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New iris value.
		@return New nearest Iris value. */
	double SetIris(double Value);
	/** Increment iris value to next possible value.
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Iris value, same as previous if max. value reached */
	double IncIris();
	/** Decrement iris value to next possible value.
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Iris value, same as previous if min. value reached */
	double DecIris();
	/** Set the iris value to the previous value divided by two
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Iris value, same as previous if min. value reached */
	double IrisHalf();
	/** Set the iris value to the previous value times two
		Note: If iris is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Iris value, same as previous if max. value reached */
	double IrisDouble();

	// ---------------------------- FOCUS --------------------------------

	/** Returns true if the camera support Focus. */
	bool FocusAvailable() const;
	/** Returns true if focus on the camera is turned on. False if off. */
	bool GetFocusOn() const;
	/** Set actual focus on value. Call with true if the camera 
		should use focus, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetFocusOn(bool onoff);
	/** Get actual focus value. Only valid if Initialize() has been
		successfully called	and FocusAvailable() is true.
		If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Focus value (no unit) */
	double GetFocus() const;
	/** Get max. focus value. Only valid if Initialize() has been
		successfully called and FocusAvailable() is true.
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Focus value. */
	double GetFocusMin() const;
	/** Get max. focus value. Only valid if Initialize() has been
		successfully called and FocusAvailable() is true.
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Focus value. */
	double GetFocusMax() const;
	/** Set focus value. If the given value is not possible the 
		nearest value are choosen.
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New focus value.
		@return New nearest Focus value. */
	double SetFocus(double Value);
	/** Increment focus value to next possible value.
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Focus value, same as previous if max. value reached */
	double IncFocus();
	/** Decrement focus value to next possible value.
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Focus value, same as previous if min. value reached */
	double DecFocus();
	/** Set the focus value to the previous value divided by two
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Focus value, same as previous if min. value reached */
	double FocusHalf();
	/** Set the focus value to the previous value times two
		Note: If focus is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Focus value, same as previous if max. value reached */
	double FocusDouble();

	// ---------------------------- GAIN ---------------------------------

	/** Returns true if the camera support Gain. */
	bool GainAvailable() const;
	/** Returns true if gain on the camera is turned on. False if off. */
	bool GetGainOn() const;
	/** Set actual gain on value. Call with true if the camera 
		should use again, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetGainOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetGainAuto() const;
	/** Set actual gain auto value. Call with true if the camera 
		should adjust gain, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetGainAuto(bool automan);
	/** Get actual gain value. Only valid if Initialize() has been
		successfully called	and GainAvailable() is true.
		If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Gain value (no unit) */
	double GetGain() const;
	/** Get max. gain value. Only valid if Initialize() has been
		successfully called and GainAvailable() is true.
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Gain value. */
	double GetGainMin() const;
	/** Get max. gain value. Only valid if Initialize() has been
		successfully called and GainAvailable() is true.
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Gain value. */
	double GetGainMax() const;
	/** Set gain value. If the given value is not possible the 
		nearest value are choosen.
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New gain value.
		@return New nearest Gain value. */
	double SetGain(double Value);
	/** Increment gain value to next possible value.
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gain value, same as previous if max. value reached */
	double IncGain();
	/** Decrement gain value to next possible value.
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gain value, same as previous if min. value reached */
	double DecGain();
	/** Set the gain value to the previous value divided by two
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gain value, same as previous if min. value reached */
	double GainHalf();
	/** Set the gain value to the previous value times two
		Note: If gain is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Gain value, same as previous if max. value reached */
	double GainDouble();
	
	// ---------------------------- SHUTTER ------------------------------

	/** Returns true if the camera support Shutter. */
	bool ShutterAvailable() const;
	/** Returns true if shutter on the camera is turned on. False if off. */
	bool GetShutterOn() const;
	/** Set actual shutter on value. Call with true if the camera 
		should use shutter, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetShutterOn(bool onoff);
	/** Returns true if the camera is in auto-mode. False if in manual mode. */
	bool GetShutterAuto() const;
	/** Set actual shutter auto value. Call with true if the camera 
		should adjust shutter, false if the software will handle it.
		@param automan Wanted mode (true=auto, false=manual)
		@return true if successfull, false otherwise */
	bool SetShutterAuto(bool automan);
	/** Get actual shutter value. Only valid if Initialize() has been
		successfully called	and ShutterAvailable() is true.
		If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Shutter value (no unit) */
	double GetShutter() const;
	/** Get max. shutter value. Only valid if Initialize() has been
		successfully called and ShutterAvailable() is true.
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Shutter value. */
	double GetShutterMin() const;
	/** Get max. shutter value. Only valid if Initialize() has been
		successfully called and ShutterAvailable() is true.
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Shutter value. */
	double GetShutterMax() const;
	/** Set shutter value. If the given value is not possible the 
		nearest value are choosen.
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New shutter value.
		@return New nearest Shutter value. */
	double SetShutter(double Value);
	/** Increment shutter value to next possible value.
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Shutter value, same as previous if max. value reached */
	double IncShutter();
	/** Decrement shutter value to next possible value.
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Shutter value, same as previous if min. value reached */
	double DecShutter();
	/** Set the shutter value to the previous value divided by two
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Shutter value, same as previous if min. value reached */
	double ShutterHalf();
	/** Set the shutter value to the previous value times two
		Note: If shutter is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Shutter value, same as previous if max. value reached */
	double ShutterDouble();

	// ---------------------------- ZOOM ---------------------------------

	/** Returns true if the camera support Zoom. */
	bool ZoomAvailable() const;
	/** Returns true if zoom on the camera is turned on. False if off. */
	bool GetZoomOn() const;
	/** Set actual zoom on value. Call with true if the camera 
		should use zoom, false if not.
		@param onoff Wanted mode (true=on, false=off)
		@return true if successfull, false otherwise */
	bool SetZoomOn(bool onoff);
	/** Get actual zoom value. Only valid if Initialize() has been
		successfully called	and ZoomAvailable() is true.
		If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Zoom value (no unit) */
	double GetZoom() const;
	/** Get max. zoom value. Only valid if Initialize() has been
		successfully called and ZoomAvailable() is true.
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Zoom value. */
	double GetZoomMin() const;
	/** Get max. zoom value. Only valid if Initialize() has been
		successfully called and ZoomAvailable() is true.
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return Max. Zoom value. */
	double GetZoomMax() const;
	/** Set zoom value. If the given value is not possible the 
		nearest value are choosen.
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@param Value New zoom value.
		@return New nearest Zoom value. */
	double SetZoom(double Value);
	/** Increment zoom value to next possible value.
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Zoom value, same as previous if max. value reached */
	double IncZoom();
	/** Decrement zoom value to next possible value.
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Zoom value, same as previous if min. value reached */
	double DecZoom();
	/** Set the zoom value to the previous value divided by two
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Zoom value, same as previous if min. value reached */
	double ZoomHalf();
	/** Set the zoom value to the previous value times two
		Note: If zoom is not available then a dummy implementation
		returning -1 is used and an error message is produced.
		@return New Zoom value, same as previous if max. value reached */
	double ZoomDouble();

	// ---------------------------- PARTIAL SCAN -------------------------

	/** Check if partial scan is available. For the firewire implementation, this method
		returns true by checking if format 7 is available. If available, user must initialize
		it by calling SetFormatAndMode(), setting Format to 7 and Mode to one of the
		supported modes for actual camera (see GetAllFormatsAndModes() for info about
		available modes for format 7).
		@return True if partial scan is available.
		@see SetFormatAndMode */
	bool PartialScanAvailable() const;
	/** Get partial scan positions. Camera must be set to format 7 before calling
		this method.
		@param UpperLeft Position in upper left corner of image.
		@param LowerRight Position in lower left corner of image.
		@return False if reading the positions failed or camera not in format 7.
		@see SetFormatAndMode */
	//bool GetPartialScan(CPoint2D<UINT16>& UpperLeft, CPoint2D<UINT16>& LowerRight) const;
	/** Set partial scan positions. Camera must be set to format 7 before calling 
		this method. Note that 1394 firewire cameras have a step size for partial
		scan resolutions, see GetPartialScanUnitSize() for more info.
		@param UpperLeft Position in upper left corner of image. Position will be rounded 
		down to nearest unit square position if value not divideable by unit size. Warning message
		will be shown.
		@param LowerRight Position in lower left corner of image. Position will be rounded 
		down to nearest unit square position if value not divideable by unit size. Warning message
		will be shown.
		@return False if the positions given are out of range or camera not in format 7.
		@see SetFormatAndMode */
	//bool SetPartialScan(const CPoint2D<UINT16>& UpperLeft,const CPoint2D<UINT16>& LowerRight);
	/** Returns the Horizontal and vertical step size. When setting partial scan,
		the width and height must be a multiplum of Horizontal and vertical step
		sizes respectively.
		@return False, if camera not in format 7 */
	//bool GetPartialScanUnitSize(unsigned int &Horizontal, unsigned int &Vertical);
	/** Returns maximum partial scan resolution for actual camera. Camera must be set
		to format 7 before calling this method.
		@return Max partial scan resolution. Returns (0,0) and produces an error if
			camera is not set to format 7. */
	//CPoint2D<UINT16> GetMaxPartialScan();
	/** Used to get information of available color codings for at given mode under 
		format 7.
		@return true if information can be acquired. Vector ColorCodings holds 
		all possible color codings. */
	bool GetColorCodingsAvailable(vector<string>& ColorCodings) const;
	/** Used to get the currently selected color coding.
		@return -1 if information can't be acquired, otherwise the color code index */
	int GetColorCoding() const;
	/** Sets the color coding for the current mode under format 7.
		@return true if operation succeeds. */
	bool SetColorCoding(unsigned int ColorCoding);


	// ---------------------------- EXTERNAL TRIGGER -------------------------
	/** Check if external trigger mode is present and supported. 
		@return True if partial scan is available. */
	bool ExternalTriggerAvailable() const;
	/** Set external trigger on or off. If trigger mode can't be turned on and off from software, 
		an error message is produced.	
		@param onOff on = true, off = false;
		@return false if operation not possible. */
	bool SetExternalTriggerOnOff(bool onOff) const;
	/** Get external trigger on or off status. 
		@return true if of, otherwise false. */
	bool GetExternalTriggerOnOffStatus() const;
	/** Set external trigger polarity. If trigger polarity can't be inverted an error message is produced.	
		@param Polarity TRUE is high-active and FALSE is low-active.
		@return false if operation not possible. */
	bool SetExternalTriggerPolarity(const bool Polarity) const;
	/** Get external trigger polarity. If trigger polarity can't be inverted an error message is produced.	
		@return true if trigger is high-active, otherwise false; */
	bool GetExternalTriggerPolarity() const;
	/** Set external trigger mode.
		@param Mode the trigger mode. Mode can be in the range [0..3]. Not all modes might be 
		available for specific camera. 
		@see GetExternalTriggerModesAvailable.
		@param N Used in som of the trigger modes. For specific use, see note below. 
		@return true if operation completed successfully.
		\verbatim
	
		Possible trigger modes are (Low-active input):
		
		Trigger mode 0: Camera starts integration of the incoming light from
                        external input falling edge. Integration time is 
                        specified by shutter value. No parameter is needed.
		
		Trigger mode 1: Camera starts integration of incoming light from 
                        external input falling edge. Integration time is 
                        equal to low state time of the external trigger 
                        input. No parameter is needed.

		Trigger mode 2: Camera starts integration of incoming light from first
                        external input falling edge. At the N-th (paramter)
                        external trigger input falling edge, integration will
                        be stopped. Parameter is required and must be two or 
                        more. (N>=2).

		Trigger mode 3: Internal trigger mode. Camera will issue trigger 
                        internally and cycle time is N times (parameter) of
                        the cycle time of the fastest frame rate. Integration
                        time of incoming light is specified by shutter value.
                        Parameter is required and must be one or more. (N>=1).

		\endverbatim
		@see IIDC 1394-based Digital Camera Specification, Version 1.30, July 25, 2000 for more details. */
	bool SetExternalTriggerMode(unsigned int Mode, int N = 0);
	/** Returns currently selected trigger mode
		@return integer in the range of [0...3]. */
	unsigned int GetExternalTriggerMode();
	/** Returns available trigger modes for camera.
		@param Modes boolean vector of size 4, each index according to one mode.
			Vector will be cleared on entry.
		@return true if operation succeded.	*/
	bool GetExternalTriggerModesAvailable(vector<bool>& Modes) const;
	/** Check if Mode is available for camera.
		@param Mode unsigned integer in the range [0..3]. 
		@see SetExternalTriggerMode;
		@return true if mode is supported. */
	bool ExternalTriggerModeAvailable(unsigned int Mode) const;
	//@}

protected: //methods
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadAutoExposure();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadShutter();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadGain();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadZoom();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadBrightness();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadGamma();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadSaturation();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadSharpness();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadWhiteBalance();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadHue();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadIris();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadFocus();
	/** Used for reading the needed value from camera to class
		@return True if successfull. */
	bool ReadFormatsModes();
	/** Reads the class version, vendor, camera name and serial number into 
		provided parameters. The is stream must point to beginning of an opened 
		setup file created with a call to Save("") or the operator<<. The 'is' 
		position will after a successfull call be set just after the value of 
		serial number.
		@return False, if the format of the setup is incorrect. */
	bool GetCameraVendorNameSerial(istream& is, double& Version, string& Vendor, 
										string& ModelName, unsigned long int& Serial) const;
	/** Returns the index in CCameraFirewireCMU::FrameRates[6] which corresponds to the
		provided FR parameter.
		@return False, if frame rate not found */
	bool GetFrameRateIndex(double FR, unsigned int& FrameRateIndex) const;
	/** Reads available formats, modes, framerates for current camera, fills in the 
		members m_FormatsAvailable, m_ModesAvailable, m_PartialScanAvailable, and
		m_PartialScanModes */
	bool ReadAvailableFormatsModes();
	/** Used for starting the ImageAcquisition
		@return True if successfull */
	bool StartImageCapture();
	/** Used for stopping the ImageAcquisition
		@return True if successfull */
	bool StopImageCapture();
	/// Returns true when StartImageCapture() has been successfully called. */
	inline bool ImageCaptureStarted() const{return m_AcquireStarted;};


};

/////////////////////////////////////////////////
//// Inline methods
/////////////////////////////////////////////////


inline bool CCameraFirewireCMU::IsCurrentModeColor() const
{
	if(!m_Initialized)
	{
		strstream ost;
		ost << "CCameraFirewireCMU::IsCurrentModeColor() Must call Initialize() first." 
			<< "\n";
		////CError::ShowMessage(IPL_ERROR,ost.str().c_str());
		return false;
	}
	if (m_BitsPerPixel==8)
		return false;
	else if (m_BitsPerPixel==24)
		return true;
	else
	{
		strstream ost;
		ost << "CCameraFirewireCMU::IsCurrentModeColor() Internal attribute m_BitsPerPixel=" 
			<< m_BitsPerPixel << " is an unknown value" <<  "\n";
		//CError::ShowMessage(IPL_ERROR,ost.str().c_str());
		return false;
	}
}

#endif //_IPL98_CAMERAFIREWIREUMC_H
