// If compiling with Visual C++, we remove some warnings
#ifdef _WIN32
#pragma warning(disable: 4786) // symbol greater than 255 character
#endif /* _WIN32 */

#include <ipl98/cpp/ipl98_general_functions.h>
#include "basler_a113.h"
#include <strstream>

using std::cerr;

namespace ipl{ // use namespace in C++

// n equals 4 bits in the following comments

// Exposure time:
//		Set through the serial port (COM) with the command :tn4n3n2n1n0
//		Read with the command :t
//		n4 is the time unit, n3-n0 is the multiplier with n3 as msb (range: 0-65535)
//		Time unit is given by:
//			n4 = x0		250 ns in each step
//			n4 = x1		500 ns in each step
//			n4 = x2		1000 ns in each step
//			n4 = x3		2000 ns in each step
//		Define maximum (MAX) exposure time for each time unit (TU)

#define TU250NSMAX 250*65535 // 16383750 ns
#define TU500NSMAX 500*65535 // 32767500 ns
#define TU1000NSMAX 1000*65535 // 65535000 ns
#define TU2000NSMAX 2000*65535 // 131070000 ns

#define SLEEPPERIOD 10 // milliseconds between succesive calls through the serial port
						// increase this value if the cable is very long (100 may be necessary)!!

// Gain:
//		Set through the serial port (COM) with the command :d0n2n1n0
//		Read with the command :d0
//		In n2 the three msb's must be 0, the remaining 9 bits gives the range: 0-511

// Offset:
//		Set through the serial port (COM) with the command :d1n2n1n0
//		Read with the command :d1
//		In n2 the two msb's must be 0, the remaining 10 bits gives the range: 0-1023

// Mode:
//		The Basler A113 has six modes, at the moment only Programmable
//		mode - fixed timing is used. Exposure steps are then 82.6 micro-seconds

#define PROGMODE_FIXEDTIMING

CBaslerA113::CBaslerA113()
{
	CCameraFGStd::CCameraFGStd(); // call base class constructor
	m_ShutterMin = 250/1000.0;
	m_ShutterMax = TU2000NSMAX/1000.0;
	m_GainMin = 0.0;
	m_GainMax = 511.0;
	m_pCom = NULL;
	m_ComPort="COM2";
}

CBaslerA113::CBaslerA113(string ComPort)
{
	CCameraFGStd::CCameraFGStd(); // call base class constructor
	m_ShutterMin = 250/1000.0;
	m_ShutterMax = TU2000NSMAX/1000.0;
	m_GainMin = 0.0;
	m_GainMax = 511.0;
	m_pCom = NULL;
	m_ComPort=ComPort;
}

CBaslerA113::~CBaslerA113()
{
	CCameraFGStd::~CCameraFGStd(); // call base class destructor
	if(m_pCom != NULL)
		delete m_pCom;
}

bool CBaslerA113::Initialize(CFrameGrabberGeneric& fg,unsigned short PortNr)
{
	// opening the com port
	m_pCom = new CCom();
	if(!m_pCom->Open(m_ComPort))
		return false;
	// initialize gain, shutter and offset by getting camera values
	GetGain();
	GetShutter();
	GetOffset();
	// Sets soft clipping to zero
	m_pCom->Write(":d2000\n");
	// call base class Initialize
	CCameraFGStd::Initialize(fg,PortNr);
	return true;
}


string CBaslerA113::GetCameraName() const
{
	return string("Basler A113");
}


bool CBaslerA113::Close()
{
	if (IsInitialized())
	{
		if(m_pCom->IsPortOpen())
		{
			if(!m_pCom->Close())
			{
				cerr << "Error: Unable to close com object" << endl;
				return false;
			}
		}
		if(m_pFrameGrabberGeneric->IsDeviceOpen())
		{
			if(!m_pFrameGrabberGeneric->Close())
			{
				cerr << "Error: Unable to close framegrabber object" << endl;
				return false;
			}
		}
		m_Initialized=false;
//		m_DeviceOpen = false;
	}
	return true;
}


bool CBaslerA113::ShutterAvailable() const
{
	return true;
}


double CBaslerA113::GetShutter() const
{
	Sleep(SLEEPPERIOD);
	m_pCom->Write(string(":t\n"));
	Sleep(SLEEPPERIOD);
	string str;
#ifndef NOCOMCONNECTION
	m_pCom->Read(str);
#endif
	Sleep(SLEEPPERIOD);
	unsigned long Value;
	sscanf(str.c_str(),"%x",&Value);

	int tu = 250*pow(2,((Value & 0x30000)/(0x10000))); // time unit in nano seconds
	double shutter = tu*((Value) & (0xffff))/1000.0; // shutter is in micro seconds

	//cout << "GetShutter(): tu=" << tu << " multiplier=" << ((Value) & (0xffff)) << " shutter=" << shutter << "micro-seconds" << endl;
	return shutter;
}


double CBaslerA113::SetShutter(double Value)
{
	Value *= 1000; // Convert Value to nano-seconds
	// Find time unit to use
	unsigned long TimeUnit=GetTU(Value);
	// the time unit bit combination 
	unsigned long TUBitComb=GetTUBitComb(TimeUnit);
	// corresponding multiplier
	unsigned long Multiplier = GetMultiplier(Value);

	// Set shutter to: TUBitComb | Multiplier
	string str(":t");
	LongToHex(TUBitComb | Multiplier,5,str);
	str.append("\n");
	m_pCom->Write(str);
	Sleep(SLEEPPERIOD);
	string s;
	m_pCom->Read(s,1);
	double ActValue=Round(Value/82600.0)*82600;
	return GetShutter();
}


double CBaslerA113::IncShutter()
{
	double Value=GetShutter();
	Value *= 1000;			// Convert Value to nano-seconds
#ifdef PROGMODE_FIXEDTIMING // exposure steps are 82.6 micro-seconds
	Value += (82.6 * 1000); // next shutter time
#endif
	return SetShutter(Value/1000.0);
}


double CBaslerA113::DecShutter()
{
	double Value=GetShutter();
	Value *= 1000;			// Convert Value to nano-seconds
#ifdef PROGMODE_FIXEDTIMING // exposure steps are 82.6 micro-seconds
	Value -= (82.6 * 1000); // next shutter time
#endif
	return SetShutter(Value/1000.0);
}


double CBaslerA113::ShutterDouble()
{
	double Value=GetShutter(); // in micro-seconds
	return SetShutter(Value*2.0);
}


double CBaslerA113::ShutterHalf()
{
	double Value=GetShutter(); // in micro-seconds
	return SetShutter(Value/2.0);
}

bool CBaslerA113::GainAvailable() const
{
	return true;
}


double CBaslerA113::GetGain() const
{
	string str;
	Sleep(SLEEPPERIOD);
	m_pCom->Write(string(":d0\n"));
	Sleep(SLEEPPERIOD);
#ifndef NOCOMCONNECTION
	m_pCom->Read(str);
#endif
	Sleep(SLEEPPERIOD);
	unsigned long Value;
	sscanf(str.c_str(),"%x",&Value);
	return Value;
}


double CBaslerA113::SetGain(double Value)
{
	if (Value<0)
	{
		cerr << "Warning: CBaslerA113::SetGain: Value<0 - setting Value to 0" << endl;
		Value=0.0;
	}

	if ((unsigned long) Value != Value)
		Value=Round(Value);

	if (Value>511)
	{
		cerr << "Warning: CBaslerA113::SetGain: Value>511 - setting Value to 511" << endl;
		Value=511;
	}
	string str(":d0");
	LongToHex((unsigned long) Value,3,str);
	str.append("\n");
	m_pCom->Write(str);
	Sleep(SLEEPPERIOD);
	string s;
	m_pCom->Read(s,1);
	return GetGain();
}


double CBaslerA113::IncGain()
{
	return SetGain(GetGain()+1);
}


double CBaslerA113::DecGain()
{
	return SetGain(GetGain()-1);
}


double CBaslerA113::GetOffset() const
{
	string str;
	Sleep(SLEEPPERIOD);
	m_pCom->Write(string(":d1\n"));
	Sleep(SLEEPPERIOD);
#ifndef NOCOMCONNECTION
	m_pCom->Read(str);
#endif
	Sleep(SLEEPPERIOD);
	unsigned long Value;
	sscanf(str.c_str(),"%x",&Value);
	double offset = (Value & 0x3ff);
	return offset;
}



double CBaslerA113::SetOffset(double Value)
{
	if (Value<0)
	{
		cerr << "Warning: CBaslerA113::SetOffset: Value<0 - setting Value to 0" << endl;
		Value=0.0;
	}

	if ((unsigned long) Value != Value)
		Value=Round(Value);

	if (Value>1023)
	{
		cerr << "Warning: CBaslerA113::SetOffset: Value>1023 - setting Value to 1023" << endl;
		Value=1023;
	}
	string str;
	str.erase();
	str = ":d1";
	LongToHex((unsigned long) Value,3,str);
	str.append("\n");
	m_pCom->Write(str);
	Sleep(SLEEPPERIOD);
	string s;
	m_pCom->Read(s,1);
	return GetOffset();
}


double CBaslerA113::IncOffset()
{
	return SetOffset(GetOffset()+1);
}


double CBaslerA113::DecOffset()
{
	return SetOffset(GetOffset()-1);
}


double CBaslerA113::GetOffsetMin() const
{
	return 0.0;
}


double CBaslerA113::GetOffsetMax() const
{
	return 1023;
}

bool CBaslerA113::ZoomAvailable() const
{
	return false;
}

bool CBaslerA113::PartialScanAvailable() const
{
	return false;
}


unsigned long CBaslerA113::GetTU(double Value)
{
	// Value is in nano-seconds
	unsigned long TimeUnit=0;
	// Find time unit
	if (Value<0.0)
	{
		// keep TimeUnit at 0 - false value
		TimeUnit=0;
		cerr << "CBaslerA113::GetTU: parameter Value <0 - error\n";
	}
	else if (Value<=TU250NSMAX)
		TimeUnit=250;
	else if (Value<=TU500NSMAX)
		TimeUnit=500;
	else if (Value<=TU1000NSMAX)
		TimeUnit=1000;
	else
		TimeUnit=2000;

	return TimeUnit;
}



unsigned long CBaslerA113::GetTUBitComb(unsigned long TUValue)
{
	unsigned long TUBitComb;
	// Find time unit to use
	if (TUValue==250)
		TUBitComb=0;
	else if (TUValue==500)
		TUBitComb=(1 << 16);
	else if (TUValue==1000)
		TUBitComb=(2 << 16);
	else if (TUValue==2000)
		TUBitComb=(3 << 16);
	else
	{
		cerr << "CBaslerA113::GetTUBitComb: parameter TUValue=" << TUValue << " - error\n";
		TUBitComb=0;
	}
	return TUBitComb;
}



unsigned long CBaslerA113::GetMultiplier(double Value)
{
	unsigned long TimeUnit=GetTU(Value);
	double Multiplier=Round(Value/TimeUnit);
	return (Multiplier>65535 ? 65535 : Multiplier);
}



bool CBaslerA113::LongToHex(unsigned long Value,unsigned char HexNumbers, string& str)
{
	if (HexNumbers>8)
	{
		cerr << "CBaslerA113::LongToHex: HexNumbers>4 not possible" << endl;
		return false;
	}
	unsigned short temp;
	char* c=(char*)malloc(2);
	for(int x=0; x<HexNumbers; x++)
	{
		temp=(unsigned short)((Value << (4*x+(32-4*HexNumbers))) >> (28));
		c[0]=GetHex(temp);
		c[1]='\0';
		str.append(c);
	}
	delete c;
	return true;
}


char CBaslerA113::GetHex(unsigned short Value)
{
	switch(Value)
	{
		case 0:  return '0';
		case 1:  return '1';
		case 2:  return '2';
		case 3:  return '3';
		case 4:  return '4';
		case 5:  return '5';
		case 6:  return '6';
		case 7:  return '7';
		case 8:  return '8';
		case 9:  return '9';
		case 10: return 'a';
		case 11: return 'b';
		case 12: return 'c';
		case 13: return 'd';
		case 14: return 'e';
		case 15: return 'f';
		default: return '?';
	}
}

} // end namespace ipl
